# 作者信息
# 作者：格蕾特（グレーテ）
# 日期：2024年12月18日
# 版本：1.0


import sys
import os
import re
import tkinter as tk
from tkinter import simpledialog
from datetime import datetime, timezone, timedelta
import pytz  # 用于处理时区

# 设置输出编码为 UTF-8
sys.stdout.reconfigure(encoding='utf-8')

# 配置功能开关
ENABLE_NOTE = True  # 控制笔记功能
ENABLE_QUERY_COUNT = True  # 控制查询次数记录功能

# 文件路径配置
NOTE_FILE = r'C:\MySoftware\GoldenDict\note\GoldenDict-MyNote.txt'
QUERY_COUNT_FILE = r'C:\MySoftware\GoldenDict\note\GoldenDict-Query.txt'

# 获取用户本地时区
local_tz = datetime.now().astimezone().tzinfo

# 获取查询词
if len(sys.argv) < 2:
    print("<html><body>Error: No word provided.</body></html>")
    sys.exit(1)

word = sys.argv[1]

# 判断查询词是否包含 edit=true
is_edit_mode = '?edit=true' in word

# 去除查询词中的 ?edit=true 部分，只在末尾出现时去除
if word.endswith("?edit=true"):
    word = word[:-10]  # 去掉末尾的 ?edit=true

# 检查文件是否存在
if not os.path.exists(NOTE_FILE):
    print(f"<html><body>Error: Note file not found: {NOTE_FILE}</body></html>")
    sys.exit(1)

from datetime import datetime, timedelta
import os

from datetime import datetime, timedelta
import os

# 记录查询次数
if ENABLE_QUERY_COUNT:
    query_data = {}

    if os.path.exists(QUERY_COUNT_FILE):
        with open(QUERY_COUNT_FILE, 'r', encoding='utf-8') as file:
            for line in file:
                parts = line.strip().split('\t')
                if len(parts) == 4:
                    q_word, count, first_time, last_time = parts
                    query_data[q_word] = {
                        'count': int(count),
                        'first_time': first_time,
                        'last_time': last_time
                    }

    now = datetime.now(local_tz).replace(microsecond=0)  # now 转换为 datetime 对象
    
    if word in query_data:
        prev_last_time = query_data[word]['last_time']
        # 计算上次查询时间与当前时间的差值
        last_time_obj = datetime.fromisoformat(prev_last_time).astimezone(local_tz)
        time_diff = now - last_time_obj

        # 如果上次查询时间与当前时间的差值小于10分钟，不增加查询次数
        if time_diff < timedelta(minutes=10):
            query_output = ""  # 不计入查询
        else:
            query_data[word]['count'] += 1
            query_data[word]['last_time'] = now.isoformat()  # 保存为 ISO 格式字符串
    else:
        prev_last_time = "N/A"
        query_data[word] = {
            'count': 1,
            'first_time': now.isoformat(),  # 保存为 ISO 格式字符串
            'last_time': now.isoformat()  # 保存为 ISO 格式字符串
        }

    # 更新文件
    with open(QUERY_COUNT_FILE, 'w', encoding='utf-8') as file:
        for q_word, data in query_data.items():
            file.write(f"{q_word}\t{data['count']}\t{data['first_time']}\t{data['last_time']}\n")

# 获取查询次数输出
if ENABLE_QUERY_COUNT and word in query_data:
    data = query_data[word]
    first_time = datetime.fromisoformat(data['first_time']).astimezone(local_tz)
    last_time = datetime.fromisoformat(data['last_time']).astimezone(local_tz)
    now = datetime.now(local_tz)

    def format_time_difference(time):
        diff = now - time
        seconds = diff.total_seconds()
        if seconds < 86400:
            if seconds < 60:
                return f"{int(seconds)} seconds ago"
            elif seconds < 3600:
                return f"{seconds / 60:.1f} minutes ago"
            else:
                return f"{seconds / 3600:.1f} hours ago"
        else:
            return time.strftime('%Y-%m-%dT%H:%M:%S%z')

    first_time_display = format_time_difference(first_time)

    if prev_last_time != "N/A":
        second_to_last_time = datetime.fromisoformat(prev_last_time).astimezone(local_tz)
        last_time_display = format_time_difference(second_to_last_time)
    else:
        last_time_display = "N/A"

    query_output = (
        f"<b>{word}</b> queried <b>{data['count']}</b> times, "
        f"first query time is <b>{first_time_display}</b>, "
        f"last query time is: <b>{last_time_display}</b>."
    )
else:
    query_output = ""


# 读取笔记文件内容
if ENABLE_NOTE:
    with open(NOTE_FILE, 'r', encoding='utf-8') as file:
        content = file.read()

    # 构建正则表达式
    pattern = rf"{re.escape(word)}\n*<font size=5>{re.escape(word)}</font>\n*<p class=\"senselevel-1\">(.*?)</p>\n*</>"

    # 查找匹配内容
    match = re.search(pattern, content, re.DOTALL)

    # 判断是否匹配到内容
    if match:
        # 提取匹配的内容
        result = match.group(1).strip()
    else:
        # 查询词不存在，设置为错误消息
        result = f"No note found for the word: {word}"

# 定义保存新内容的方法
if ENABLE_NOTE:
    def save_content(new_content):
        global content

        # 将换行符前补充 <br/> 标签，但保留原来的换行符
        new_content = new_content.replace('\n', '<br/>\n')

        # 如果笔记为空，则删除该词条
        if new_content.strip() == "":
            # 如果词条存在，删除该词条
            if match:
                old_entry = match.group(0)
                content = content.replace(f"{old_entry}\n", "")  # 删除该条目
        else:
            # 如果词条存在，替换旧内容
            if match:
                old_entry = match.group(0)
                new_entry = f"{word}\n<font size=5>{word}</font>\n<p class=\"senselevel-1\">{new_content}</p>\n</>"
                content = content.replace(old_entry, new_entry)
            else:
                # 如果词条不存在，按正确格式追加新内容
                new_entry = f"{word}\n<font size=5>{word}</font>\n<p class=\"senselevel-1\">{new_content}</p>\n</>"
                content += f"\n{new_entry}"

        # 写入文件
        with open(NOTE_FILE, 'w', encoding='utf-8') as file:
            file.write(content)

        # 关闭 Tkinter 窗口
        root.destroy()

        # 更新浏览器 URL（使用 JavaScript 重定向）
        print('<script>window.location.replace("' + word + '");</script>')

# 定义编辑界面
if ENABLE_NOTE:
    def edit_interface():
        global root  # 确保可以关闭 root 窗口
        root = tk.Tk()
        root.title(f"Edit Note for '{word}'")

        # 创建可调整大小的框架
        frame = tk.Frame(root)
        frame.pack(fill=tk.BOTH, expand=True, padx=10, pady=10)

        # 标签
        tk.Label(frame, text=f"Current Note for '{word}':").pack(pady=5)

        # 启用撤销功能的 Text 小部件
        text_area = tk.Text(frame, wrap=tk.WORD, undo=True)
        text_area.pack(fill=tk.BOTH, expand=True)

        # 自动聚焦到文本区域
        text_area.focus_set()

        # 填充现有内容
        if match:
            # 将 <br/> 标签替换为换行符
            result_for_text_area = result.replace('<br/>', '')
            text_area.insert("1.0", result_for_text_area)

        # 保存按钮
        def on_save():
            new_content = text_area.get("1.0", "end").strip()
            save_content(new_content)

        # 按钮框架
        button_frame = tk.Frame(frame)
        button_frame.pack(fill=tk.X, pady=10)

        # 保存按钮
        tk.Button(button_frame, text="Save", command=on_save).pack(side=tk.LEFT, padx=5)

        # 绑定 Ctrl + S 快捷键触发保存
        root.bind('<Control-s>', lambda event: on_save())

        # 绑定 Ctrl + Z 快捷键触发撤销
        def on_undo(event=None):
            try:
                text_area.edit_undo()  # 调用 Text 小部件的撤销方法
            except tk.TclError:
                pass  # 如果没有更多可撤销的操作，忽略错误

        root.bind('<Control-z>', on_undo)  # 绑定 Ctrl+Z 快捷键


        
        # 绑定窗口关闭事件
        def on_close():
            # 当用户关闭窗口时，执行与保存相同的操作
            on_save()
        
        root.protocol("WM_DELETE_WINDOW", on_close)  # 绑定窗口关闭按钮

        # 以下是 B, I, U, S 按钮的功能实现

        # 在选中的文本两端插入 HTML 标签，如果没有选中文本则插入标签
        def insert_tag(tag):
            start_idx = text_area.index(tk.SEL_FIRST) if text_area.tag_ranges(tk.SEL) else text_area.index(tk.INSERT)
            end_idx = text_area.index(tk.SEL_LAST) if text_area.tag_ranges(tk.SEL) else start_idx

            selected_text = text_area.get(start_idx, end_idx)

            # 如果有选中文本
            if selected_text:
                text_area.delete(start_idx, end_idx)
                text_area.insert(start_idx, f"<{tag}>{selected_text}</{tag}>")
            else:
                text_area.insert(start_idx, f"<{tag}></{tag}>")
                text_area.mark_set(tk.INSERT, start_idx)

        # 添加 B 按钮（粗体）
        def add_bold():
            insert_tag('b')

        # 添加 I 按钮（斜体）
        def add_italic():
            insert_tag('i')

        # 添加 U 按钮（下划线）
        def add_underline():
            insert_tag('u')

        # 添加 S 按钮（删除线）
        def add_strike():
            insert_tag('s')

        # 添加 专名 按钮（带下划线的文本）
        def add_proper_name():
            start_idx = text_area.index(tk.SEL_FIRST) if text_area.tag_ranges(tk.SEL) else text_area.index(tk.INSERT)
            end_idx = text_area.index(tk.SEL_LAST) if text_area.tag_ranges(tk.SEL) else start_idx

            selected_text = text_area.get(start_idx, end_idx)

            # 如果有选中文本
            if selected_text:
                text_area.delete(start_idx, end_idx)
                text_area.insert(start_idx, f'<span style="border-bottom: 1px solid; margin: 0 1px;">{selected_text}</span>')
            else:
                text_area.insert(start_idx, '<span style="border-bottom: 1px solid; margin: 0 1px;"></span>')
                text_area.mark_set(tk.INSERT, start_idx)

        # 添加 书名 按钮（带波浪下划线的文本）
        def add_book_title():
            start_idx = text_area.index(tk.SEL_FIRST) if text_area.tag_ranges(tk.SEL) else text_area.index(tk.INSERT)
            end_idx = text_area.index(tk.SEL_LAST) if text_area.tag_ranges(tk.SEL) else start_idx

            selected_text = text_area.get(start_idx, end_idx)

            # 如果有选中文本
            if selected_text:
                text_area.delete(start_idx, end_idx)
                text_area.insert(start_idx, f'<span style="text-decoration: underline; text-decoration-style: wavy; text-decoration-thickness: 1px; text-underline-offset: 3px; margin: 0 1px;">{selected_text}</span>')
            else:
                text_area.insert(start_idx, '<span style="text-decoration: underline; text-decoration-style: wavy; text-decoration-thickness: 1px; text-underline-offset: 3px; margin: 0 1px;"></span>')
                text_area.mark_set(tk.INSERT, start_idx)

        # 添加 href 按钮（超链接）
        def add_href():
            start_idx = text_area.index(tk.SEL_FIRST) if text_area.tag_ranges(tk.SEL) else text_area.index(tk.INSERT)
            end_idx = text_area.index(tk.SEL_LAST) if text_area.tag_ranges(tk.SEL) else start_idx

            selected_text = text_area.get(start_idx, end_idx)

            # 如果有选中文本
            if selected_text:
                text_area.delete(start_idx, end_idx)
                text_area.insert(start_idx, f'<a href="entry://{selected_text}#section">{selected_text}</a>')
            else:
                text_area.insert(start_idx, '<a href="entry://示例文字#section">示例文字</a>')
                text_area.mark_set(tk.INSERT, start_idx)

        # 创建 B, I, U, S, 专名, 书名, href 按钮
        tk.Button(root, text="B", command=add_bold).pack(side=tk.LEFT, padx=5, pady=5)
        tk.Button(root, text="I", command=add_italic).pack(side=tk.LEFT, padx=5, pady=5)
        tk.Button(root, text="U", command=add_underline).pack(side=tk.LEFT, padx=5, pady=5)
        tk.Button(root, text="S", command=add_strike).pack(side=tk.LEFT, padx=5, pady=5)
        tk.Button(root, text="专名", command=add_proper_name).pack(side=tk.LEFT, padx=5, pady=5)
        tk.Button(root, text="书名", command=add_book_title).pack(side=tk.LEFT, padx=5, pady=5)
        tk.Button(root, text="href", command=add_href).pack(side=tk.LEFT, padx=5, pady=5)

        root.mainloop()

# 输出 HTML 格式
if ENABLE_NOTE:
    output_html = f"""
    <html>
    <body>
    <p>{query_output}</p>
    <p>{result}</p>
    <a href="{word}?edit=true" id="edit_link">Edit my note</a>
    </body>
    </html>
    """
else:
    output_html = f"""
    <html>
    <body>
    <p>{query_output}</p>
    </body>
    </html>
    """


# 如果查询词包含 ?edit=true，启动 Tkinter 编辑窗口
if is_edit_mode and ENABLE_NOTE:
    edit_interface()
else:
    # 输出 HTML 内容
    print(output_html)
